@@ -6,7 +6,7 @@
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
- * $Header: /cvsroot/pgsql-server/src/bin/pg_dump/pg_dumpall.c,v 1.9.2.1 2002/11/22 03:09:59 tgl Exp $
+ * $Header: /cvsroot/pgsql/src/bin/pg_dump/pg_dumpall.c,v 1.9.2.4 2003/03/14 22:45:56 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -77,6 +77,7 @@ main(int argc, char *argv[])
 		{"inserts", no_argument, NULL, 'd'},
 		{"attribute-inserts", no_argument, NULL, 'D'},
 		{"column-inserts", no_argument, NULL, 'D'},
+		{"globals-only", no_argument, NULL, 'g'},
 		{"host", required_argument, NULL, 'h'},
 		{"ignore-version", no_argument, NULL, 'i'},
 		{"oids", no_argument, NULL, 'o'},
@@ -344,7 +345,7 @@ dumpGroups(PGconn *conn)
 
 		val = strdup(PQgetvalue(res, i, 2));
 		tok = strtok(val, ",{}");
-		do
+		while (tok)
 		{
 			PGresult   *res2;
 			PQExpBuffer buf2 = createPQExpBuffer();
@@ -364,7 +365,7 @@ dumpGroups(PGconn *conn)
 
 			tok = strtok(NULL, "{},");
 		}
-		while (tok);
+		free(val);
 
 		printf("%s", buf->data);
 		destroyPQExpBuffer(buf);
@@ -414,7 +415,7 @@ dumpCreateDB(PGconn *conn)
 			continue;
 
 		if (output_clean)
-			appendPQExpBuffer(buf, "DROP DATABASE %s\n;", fmtId(dbname));
+			appendPQExpBuffer(buf, "DROP DATABASE %s;\n", fmtId(dbname));
 
 		appendPQExpBuffer(buf, "CREATE DATABASE %s", fmtId(dbname));
 		appendPQExpBuffer(buf, " WITH OWNER = %s TEMPLATE = template0", fmtId(dbowner));
